
# q-expansion of newform 370.2.a.f, downloaded from the LMFDB on 27 April 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 370
weight = 2
poly_data = [-8, -1, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[1, 0], [2, 0], [1, 0], [-1, -1], [-1, 1], [0, 2], [-3, 1], [-2, 0], [2, -2], [1, -3], [-5, -1], [1, 0], [3, -1], [5, -1], [-4, 2], [-1, -1], [8, -2], [-3, 1], [0, -2], [-2, 2], [4, -2], [0, 2], [4, 2], [10, 0], [7, 3], [-2, 4], [0, 4], [6, 4], [-7, -3], [15, -1], [-12, 2], [-8, -2], [12, -2], [-9, -3], [-2, 4], [-20, 0], [15, -1], [17, -1], [-8, -4], [11, -5], [-12, 6], [-2, -4], [21, 1], [2, 0], [18, -4], [-16, -2], [-7, 3], [1, 1], [-9, 1], [-10, 0], [-8, -6], [1, -7], [4, 6], [2, -4], [-2, 8], [-7, -3], [10, 0], [-2, 2], [-4, -2], [20, -2], [-4, -4], [-7, 5], [-14, 4], [1, 1], [6, 4], [23, -1], [14, -4], [-12, -2], [-4, 8], [-22, 0], [-11, 5], [-12, 8], [11, 3], [-22, 4], [-8, 0], [-4, 4], [3, -5], [6, -8], [10, 0], [-8, -6], [4, -4], [2, 4], [1, 1], [-6, -8], [-11, 1], [2, 8], [18, 0], [7, -5], [-13, 11], [4, 4], [9, -9], [8, 2], [-14, 10], [-4, -8], [-6, 8], [20, -4], [-16, -2], [1, -11], [28, 0], [30, 0], [37, -1], [4, -2], [7, -11], [-14, 8], [-11, 7], [2, 4], [17, -13], [4, 6], [8, -4], [-3, 1], [-6, -10], [17, -11], [-8, 10], [23, -3], [7, 11], [-33, 7], [1, -1], [-4, 4], [10, 4], [12, 0], [-17, -9], [-18, -4], [-42, 0], [11, 1], [1, -9], [2, 12], [33, -3], [-30, 2], [-48, 0], [-27, 1], [-25, 5], [-3, 1], [6, -2], [6, -16], [3, -1], [32, -6], [-3, 9], [-22, 0], [-18, 4], [-42, 4], [40, -4], [-28, 2], [-16, 10], [-33, 9], [-19, 9], [-20, 4], [26, -4], [-41, -5], [4, 10], [23, -13], [47, -1], [-1, -1], [-9, 1], [-23, 1], [12, -12], [32, 6], [-16, -10], [5, -11], [18, -8], [18, 0], [11, -11], [32, -6], [-28, 0], [-3, 7], [-19, -3], [19, -5], [9, -7], [12, -6]]
